@@ -1,14 +1,15 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 """Support for instances and associated abstractions."""
 
 __author__ = "Lluís Vilanova"
-__copyright__ = "Copyright 2008-2019, Lluís Vilanova"
+__copyright__ = "Copyright 2008-2024, Lluís Vilanova"
 __license__ = "GPL version 3 or later"
 
 
 import collections
+import contextlib
 import pickle
 import weakref
 
@@ -20,6 +21,8 @@ from . import pp
 
 _EXPORT = [set()]
 
+_INVALID = object()
+
 
 class Instance(dict, pp.Pretty):
     """A map from variable names to values."""
@@ -36,6 +39,32 @@ class Instance(dict, pp.Pretty):
             dict.__init__(self)
         else:
             dict.__init__(self, source)
+        setattr(self, "__translation_cache", {})
+        setattr(self, "__translation_cache_enabled", False)
+
+    @contextlib.contextmanager
+    def auto_translation(self):
+        setattr(self, "__translation_cache_enabled", True)
+        yield
+        setattr(self, "__translation_cache_enabled", False)
+
+    def __getitem__(self, key):
+        if not getattr(self, "__translation_cache_enabled"):
+            return dict.__getitem__(self, key)
+
+        cache = getattr(self, "__translation_cache")
+        val = cache.get(key, _INVALID)
+        if val is _INVALID:
+            val = dict.__getitem__(self, key)
+            if isinstance(val, str):
+                from . import text
+                val = text.translate(val, self, recursive=True)
+            cache[key] = val
+        return val
+
+    def __setitem__(self, key, val):
+        getattr(self, "__translation_cache").clear()
+        return dict.__setitem__(self, key, val)
 
     def __hash__(self):
         """Hashing function for adding into `InstanceGroup`."""
@@ -75,7 +104,7 @@ class Instance(dict, pp.Pretty):
     def __getstate__(self):
         # pylint: disable=not-callable
         progress.Stack.CURRENT()
-        if not _EXPORT[-1] == 0:
+        if not _EXPORT[-1]:
             odict = self.__dict__
         else:
             odict = {}
@@ -87,12 +116,13 @@ class Instance(dict, pp.Pretty):
     def __reduce__(self):
         # pylint: disable=not-callable
         progress.Stack.CURRENT()
-        if not _EXPORT[-1] == 0:
+        if not _EXPORT[-1]:
             odict = dict(self)
         else:
             odict = {}
-            for key, val in self.items():
-                if key in _EXPORT[-1]:
+            for key in _EXPORT[-1]:
+                val = self.get(key, _INVALID)
+                if val is not _INVALID:
                     odict[key] = val
         return tuple([Instance, tuple(), odict])
 
@@ -507,7 +537,7 @@ class InstanceGroup(pp.Pretty):
                 raise ValueError("Cannot use proactive caching when "
                                  "caching is disabled")
             if not isinstance(proactive, bool) and (
-                    not isinstance(proactive, collections.Iterable) or
+                    not isinstance(proactive, collections.abc.Iterable) or
                     not all([isinstance(elem, str) for elem in proactive])):
                 raise TypeError("Invalid `proactive` caching "
                                 "value: %s" % proactive)
@@ -516,7 +546,7 @@ class InstanceGroup(pp.Pretty):
                 raise ValueError(
                     "Cannot use 'others' without proactive caching")
             if not isinstance(others, bool) and (
-                    not isinstance(others, collections.Iterable) or
+                    not isinstance(others, collections.abc.Iterable) or
                     not all([isinstance(elem, str) for elem in others])):
                 raise TypeError("Invalid `others` caching "
                                 "value: %s" % others)
