@@ -1,10 +1,10 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 """Common utility classes and functions."""
 
 __author__ = "Lluís Vilanova"
-__copyright__ = "Copyright 2008-2020, Lluís Vilanova"
+__copyright__ = "Copyright 2008-2023, Lluís Vilanova"
 __license__ = "GPL version 3 or later"
 
 
@@ -141,12 +141,12 @@ class ViewError(Exception):
         Exception.__init__(self, *args, **kwargs)
 
 
-class OrderedSet(collections.MutableSet, pp.Pretty):
+class OrderedSet(collections.abc.MutableSet, pp.Pretty):
     """A mutable set preserving order of insertion.
 
     .. todo::
 
-       All help should come from `~collections.MutableSet` instead of
+       All help should come from `~collections.abc.MutableSet` instead of
        using `_wraps`.
 
     """
@@ -349,7 +349,7 @@ class OrderedSet(collections.MutableSet, pp.Pretty):
             except ViewError:
                 self._set.remove(key)
                 raise
-    add.__doc__ = collections.MutableSet.add.__doc__
+    add.__doc__ = collections.abc.MutableSet.add.__doc__
 
     # pylint: disable=missing-docstring
     def discard(self, key):
@@ -361,11 +361,11 @@ class OrderedSet(collections.MutableSet, pp.Pretty):
             except ViewError:
                 self._set.add(key)
                 raise
-    discard.__doc__ = collections.MutableSet.discard.__doc__
+    discard.__doc__ = collections.abc.MutableSet.discard.__doc__
     discard.__doc__ += "\n\nThis operation has a cost of O(n)."
 
     # pylint: disable=missing-docstring
-    @_wraps(collections.MutableSet.pop)
+    @_wraps(collections.abc.MutableSet.pop)
     def pop(self, last=True):
         if not self:
             raise KeyError('set is empty')
