@@ -1,10 +1,10 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 """Automate generation of parametrized launchers."""
 
 __author__ = "Lluís Vilanova"
-__copyright__ = "Copyright 2008-2019, Lluís Vilanova"
+__copyright__ = "Copyright 2008-2024, Lluís Vilanova"
 __license__ = "GPL version 3 or later"
 
 
@@ -77,7 +77,7 @@ def _get_translate(experiments):
 
 def _do_execute(experiments, cmd, **kwargs):
     if len(experiments) == 0:
-        experiments_list = [{}]
+        experiments_list = [Instance()]
     else:
         experiments_list = experiments
 
@@ -130,11 +130,11 @@ def _do_execute(experiments, cmd, **kwargs):
             if repeat == "warn":
                 warnings.warn(
                     "Repeated command translation for experiments %r and %r" % (
-                        exp, e))
+                        exp, exp_rest[0]))
             elif repeat == "error":
                 raise RuntimeError(
                     "repeated command translation for experiments %r and %r" % (
-                        exp, e))
+                        exp, exp_rest[0]))
 
 
         def get_stdfile(name, env):
@@ -145,7 +145,8 @@ def _do_execute(experiments, cmd, **kwargs):
             if translator is None:
                 return as_context(std_args[name])
             else:
-                path = translator.translate(env)
+                with env.auto_translation():
+                    path = translator.translate(env)
                 return open(path, "w")
 
         with get_stdfile("stdin",  exp) as stdin,\
@@ -323,7 +324,7 @@ def _do_recombine(experiments, exp_filters, variables, var_filters, append):
 
 def _do_pack(experiments, template_from, template_to, dereference):
     if len(experiments) == 0:
-        experiments_list = [{}]
+        experiments_list = [Instance()]
     else:
         experiments_list = experiments
     if dereference is None:
@@ -336,28 +337,29 @@ def _do_pack(experiments, template_from, template_to, dereference):
     done_exp = {}
     with progress.get(len(experiments_list), msg="packing files...") as progr:
         for exp in experiments_list:
-            path_from = utils.get_path(translate_from.translate(exp))
-            path_to = utils.get_path(translate_to.translate(exp))
-            path_to = os.path.join(experiments.out, path_to)
+            with exp.auto_translation():
+                path_from = utils.get_path(translate_from.translate(exp))
+                path_to = utils.get_path(translate_to.translate(exp))
+                path_to = os.path.join(experiments.out, path_to)
 
-            done_idx = path_from + "+" + path_to
-            if done_idx in done_first:
-                if done_first[done_idx]:
-                    done_first[done_idx] = False
-                    progress.info(
-                        "Skipping already packed file: %s -> %s",
-                        path_from, path_to)
-                progress.verbose(
-                    "Conflicting experiments during pack:\n%s\n%s",
-                    done_exp[done_idx], exp)
-            else:
-                done_first[done_idx] = True
-                done_exp[done_idx] = exp
+                done_idx = path_from + "+" + path_to
+                if done_idx in done_first:
+                    if done_first[done_idx]:
+                        done_first[done_idx] = False
+                        progress.info(
+                            "Skipping already packed file: %s -> %s",
+                            path_from, path_to)
+                    progress.verbose(
+                        "Conflicting experiments during pack:\n%s\n%s",
+                        done_exp[done_idx], exp)
+                else:
+                    done_first[done_idx] = True
+                    done_exp[done_idx] = exp
 
-                progress.debug("Packing: %s -> %s", path_from, path_to)
-                utils.copy_path(path_from, path_to)
+                    progress.debug("Packing: %s -> %s", path_from, path_to)
+                    utils.copy_path(path_from, path_to)
 
-            progr()
+                progr()
 
 
 def _do_translate(experiments, exp_filters, exp_translate,
@@ -415,17 +417,18 @@ def _do_translate(experiments, exp_filters, exp_translate,
                 return translations
 
     for exp in experiments.select(exp_filters):
-        try:
-            is_match = exp_filters.match(exp)
-        except NameError:
-            is_match = False
+        with exp.auto_translation():
+            try:
+                is_match = exp_filters.match(exp)
+            except NameError:
+                is_match = False
 
-        if is_match:
-            _WithExp._EXPERIMENT = exp
-            _WithExpTpl._EXPERIMENT = exp
-            translated = [template.translate(exp)
-                          for template in parsed_templates]
-            add(translated, exp)
+            if is_match:
+                _WithExp._EXPERIMENT = exp
+                _WithExpTpl._EXPERIMENT = exp
+                translated = [template.translate(exp)
+                              for template in parsed_templates]
+                add(translated, exp)
 
     return list(result())
 
@@ -439,43 +442,47 @@ def _do_generate(experiments, template_from, template_to,
     done_exp = {}
     with progress.get(len(experiments), msg="generating files...") as progr:
         for exp in experiments:
-            path_from = utils.get_path(translate_from.translate(exp))
-            path_to = utils.get_path(translate_to.translate(exp))
-            path_to = os.path.join(experiments.out, path_to)
+            with exp.auto_translation():
+                _WithExp._EXPERIMENT = exp
+                _WithExpTpl._EXPERIMENT = exp
 
-            done_idx = path_from + "+" + path_to
-            if done_idx in done_first:
-                if done_first[done_idx]:
-                    done_first[done_idx] = False
-                    progress.info(
-                        "Skipping already generated file: %s -> %s",
-                        path_from, path_to)
-                progress.verbose(
-                    "Conflicting experiments during generation:\n%s\n%s",
-                    done_exp[done_idx], exp)
-            else:
-                done_first[done_idx] = True
-                done_exp[done_idx] = exp
+                path_from = utils.get_path(translate_from.translate(exp))
+                path_to = utils.get_path(translate_to.translate(exp))
+                path_to = os.path.join(experiments.out, path_to)
 
-                progress.debug("Generating: %s -> %s", path_from, path_to)
-
-                exp, contents_to = pre_exp_fun(exp, path_from, path_to)
-
-                utils.assert_dir(os.path.dirname(path_to))
-                if os.path.isfile(path_to):
-                    with open(path_to, "r") as f:
-                        contents_to_old = f.read()
+                done_idx = path_from + "+" + path_to
+                if done_idx in done_first:
+                    if done_first[done_idx]:
+                        done_first[done_idx] = False
+                        progress.info(
+                            "Skipping already generated file: %s -> %s",
+                            path_from, path_to)
+                    progress.verbose(
+                        "Conflicting experiments during generation:\n%s\n%s",
+                        done_exp[done_idx], exp)
                 else:
-                    contents_to_old = ""
+                    done_first[done_idx] = True
+                    done_exp[done_idx] = exp
 
-                if contents_to != contents_to_old:
-                    with open(path_to, "w") as f:
-                        f.write(contents_to)
-                shutil.copymode(path_from, path_to)
+                    progress.debug("Generating: %s -> %s", path_from, path_to)
 
-                post_exp_fun(exp, path_to)
+                    exp, contents_to = pre_exp_fun(exp, path_from, path_to)
 
-            progr()
+                    utils.assert_dir(os.path.dirname(path_to))
+                    if os.path.isfile(path_to):
+                        with open(path_to, "r") as f:
+                            contents_to_old = f.read()
+                    else:
+                        contents_to_old = ""
+
+                    if contents_to != contents_to_old:
+                        with open(path_to, "w") as f:
+                            f.write(contents_to)
+                    shutil.copymode(path_from, path_to)
+
+                    post_exp_fun(exp, path_to)
+
+                progr()
 
 def _do_generate_user(experiments, template_from, template_to):
     def pre_exp_fun(exp, path_from, path_to):
@@ -491,6 +498,18 @@ def _do_generate_jobs(experiments, system, template_to, export, depends,
                       submit_args, job_desc):
     template = templates.get(system)
 
+    depends = set(depends) | set(["{{LAUNCHER}}"])
+
+    # export all variables in depends
+    export = set(export)
+    for elem in depends:
+        if not isinstance(elem, six.string_types):
+            raise TypeError("depends element is not a string: %r" % elem)
+        for elem_var in text.get_variables(elem, nested="all"):
+            if elem_var == "LAUNCHER":
+                continue
+            export.add(elem_var)
+
     if isinstance(experiments, ExperimentsView):
         experiments_vars = list(experiments._base._experiments.variables())
     else:
@@ -500,12 +519,7 @@ def _do_generate_jobs(experiments, system, template_to, export, depends,
             raise TypeError("export element is not a string: %r" % export_elem)
         if export_elem not in experiments_vars:
             raise ValueError("missing variable to export: %s" % export_elem)
-    export = set(export) | set(template.system.assumes())
-
-    depends = set(depends) | set(["{{LAUNCHER}}"])
-    for elem in depends:
-        if not isinstance(elem, six.string_types):
-            raise TypeError("depends element is not a string: %r" % elem)
+    export = export | set(template.system.assumes())
 
     out = os.path.abspath(utils.get_path(experiments.out))
     job_desc = os.path.join(out, job_desc)
@@ -538,8 +552,6 @@ def _do_generate_jobs(experiments, system, template_to, export, depends,
             if key not in exp:
                 exp[key] = val
 
-        _WithExp._EXPERIMENT = exp
-        _WithExpTpl._EXPERIMENT = exp
         contents_to = translate_from.translate(exp)
 
         return exp, contents_to
@@ -553,8 +565,6 @@ def _do_generate_jobs(experiments, system, template_to, export, depends,
             val = exp[var]
             if not _val_is_translate(val):
                 continue
-            _WithExp._EXPERIMENT = exp
-            _WithExpTpl._EXPERIMENT = exp
             exp[var] = text.translate(val, exp)
         experiments_generated.add(Instance(exp))
 
@@ -979,29 +989,39 @@ class Experiments(pp.Pretty):
         job_desc : str, optional
             Name of the output job descriptor file.
 
-        The job descriptor will contain all variables used in `to_expr` (to
-        identify the jobs by the parameters used to generate their files) and
-        those listed in `export` (to provide further identification variables
-        established by the user).
+        All job systems implicitly define the *LAUNCHER* variable, which has the
+        path of the to the generated script to run that. All job systems also
+        define *DONE* and *FAIL* as file paths to identify the state of each
+        job.
+
+        The variables selected by `export` will be later available when using
+        `sciexp2.expdef.launcher` with this job descriptor. Any variable
+        directly or indirectly used in `depends` will be implicitly added to
+        `export`. Variable *LAUNCHER* is also implicitly added.
+
+        The templates specified by `depends` will be translated for each job
+        info file paths, and each file path will be considered a dependency of
+        that job. The path specified by *LAUNCHER* is implicitly added as a
+        dependency.
+
+        The state for a given job is decided as follows:
+
+        - Not run: Neither the *DONE* nor the *FAIL* files exist.
+        - Failed: The *FAIL* file exists.
+        - Outdated: The *DONE* file exists, but is older than any of its
+          dependencies.
+        - Done: The *DONE* file exists, and is newer than any of its
+          dependencies.
+        - Running: Some job systems can track when a job is currently executing.
+
 
         Notes
         -----
-        None of the experiments can define the *LAUNCHER* variable, which will
-        point to the generated job script for that experiment. Some job systems
-        also define their own set of reserved variables and default values to
-        others (in case your experiments don't specify them); see the job
-        system's documentation for more information.
 
-        The variables selected by `export` will be available to use in filters
-        to select the generated jobs with `sciexp2.expdef.launcher`, and for
-        templates used by other arguments to this function.
-
-        The templates specified by `depends` will be translated for each job to
-        decide if it is outdated; if one file in `depends` does not exist or has
-        been updated since last running a job, the job will be considered for
-        re-execution. The value of the *LAUNCHER* variable is always a
-        dependency (i.e., updating the contents of a job script will make it
-        appear as outdated).
+        None of the experiments can define the *LAUNCHER* variable. Some job
+        systems also define their own set of reserved variables and default
+        values to others (in case your experiments don't specify them); see the
+        job system's documentation for more information.
 
         """
         _do_generate_jobs(self, system, template_to, export, depends,
@@ -1308,6 +1328,7 @@ def with_exp_tpl(func, template):
     --------
     In its basic form, we can combine translated templates with other experiment
     values:
+
     >>> e = Experiments()
     >>> def fun(exp, tpl):
     ...     return tpl + " == " + str(exp['a']+exp['b'])
@@ -1318,6 +1339,7 @@ def with_exp_tpl(func, template):
 
     But this comes in handy when we want to pull the contents of per-experiment
     files into new experiment values. Let's start by creating some files:
+
     >>> import tempfile
     >>> tmp_dir = tempfile.TemporaryDirectory()
     >>> with open(os.path.join(tmp_dir.name, "file-0"), "w") as f:
@@ -1328,6 +1350,7 @@ def with_exp_tpl(func, template):
     3
 
     And now we can use them to populate experiment values:
+
     >>> def get_file(exp, path):
     ...    return open(path, "r").read()
     >>> v = e.view("b == 0")
